<?php
/**
 * @category Library
 * @package Drupal MVC Example
 * @author Dmitry Sheiko <me@dsheiko.com>
 * @copyright 2010-2011 Dmitry Sheiko
 * @license GNU
 * @version $Revision$
 * @link http://dsheiko.com
 */

class Lib_Application
{
    const TPL_DELIMITER = "-";
    const URI_PART_DELIMITER = '-';
    const URI_DELIMITER = '/';
    const CLASS_NAME_DELIMITER = "_";
    const CLASS_WRAPPER_TPL = "Controller_%s";
    const METHOD_WRAPPER_TPL = "%sAction";

    /**
     * Helper to wrap method name
     * @param string $name
     * @return string
     */
    private function _wrapMethod($name)
    {
        return sprintf(self::METHOD_WRAPPER_TPL, $name);
    }
    /**
     * Helper to wrap class name
     * @param string $name
     * @return string
     */
    private function _wrapClass($name)
    {
        return sprintf(self::CLASS_WRAPPER_TPL, $name);
    }
    /**
     * Helps to tranform array(part, part, part) into array(Part, Part, Part)
     * or array(part-part, part, part) into array(PartPart, Part, Part)
     * @param mixed $item
     * @param int $index
     * @param boolean $ommitFirst user parameter
     */
    static private function _ucFirstCb(&$item, $index)
    {
        $item = ucfirst($item);
        // Now check for array(part-part, part, part)
        if (strpos($item, self::URI_PART_DELIMITER) !== false) {
            $parts = explode(self::URI_PART_DELIMITER, $item);
            array_walk($parts, array("Lib_Application", "_ucFirstCb"));            
            $item = implode("", $parts);
        }
    }
    /**
     * The router is responsible for mapping
     * the template suggestions to a controllers and actions
     * .
     * @param array $templates
     * @return array of Lib_Dispatcher
     */
    public function routeTemplates(array $templates)
    {
        $res = array();
        if (!$templates) {
            throw new Lib_Application_Exception("There is nothing to route");
        }
        foreach ($templates as $template) {
            $parts = explode(self::TPL_DELIMITER, $template);
            if (($parts[0] == 'page' && count($parts) > 2) // requests like page-news-view
                || ($parts[0] != 'page' && count($parts) > 1)) { // AJAX requests like widget-some..
                $action = array_pop($parts);
            } else {
                // Action not defined, thus the default action is used
                $action =  "index";
            }
            array_walk($parts, array("Lib_Application", "_ucFirstCb"));
            $res[] = new Lib_Dispatcher(
                $this->_wrapClass(implode("", $parts))
                , $this->_wrapMethod($action));
        }
        return $res;
    }
    /**
     * The router is responsible for mapping
     * the request to a controller and action
     *
     * @param string $request
     * @return Lib_Dispatcher
     */
    public function routeRequest($request)
    {        
        $parts = explode(self::URI_DELIMITER, $request);
        array_walk($parts, array("Lib_Application", "_ucFirstCb"));        
        $action = array_pop($parts);
        $action[0] = strtolower($action[0]);
        return new Lib_Dispatcher(
            $this->_wrapClass(implode(self::CLASS_NAME_DELIMITER, $parts))
            , $this->_wrapMethod($action));
    }
    
    /**
     * Runs application: either page or requsted module
     * 
     * @param View_View $view
     * @param string $request
     * @param mixed $params
     * @return stdObject
     */
    public function run(&$view, $request = null, $params = null)
    {
        if ($request == null) {
            $map = $this->routeTemplates($view->template_files);
            foreach ($map as $call) {
               $call->dispatch($view, $params);
            }
        } else {
            return $this->routeRequest($request)->dispatch($view, $params);
        }
        return null;
    }   
}