<?php
/**
 * @category Library
 * @package Drupal MVC Example
 * @author Dmitry Sheiko <me@dsheiko.com>
 * @copyright 2010-2011 Dmitry Sheiko
 * @license GNU
 * @version $Revision$
 * @link http://dsheiko.com
 */

class Lib_Request
{

    /**
     * Return the value of the given HTTP header. Pass the header name as the
     * plain, HTTP-specified header name. Ex.: Ask for 'Accept' to get the
     * Accept header, 'Accept-Encoding' to get the Accept-Encoding header.
     *
     * @param string $header HTTP header name
     * @return string|false HTTP header value, or false if not found
     * @throws Zend_Controller_Request_Exception
     */
    static function getHeader($header)
    {
        if (empty($header)) {
            return false;
        }

        // Try to get it from the $_SERVER array first
        $temp = 'HTTP_' . strtoupper(str_replace('-', '_', $header));
        if (!empty($_SERVER[$temp])) {
            return $_SERVER[$temp];
        }

        // This seems to be the only way to get the Authorization header on
        // Apache
        if (function_exists('apache_request_headers')) {
            $headers = apache_request_headers();
            if (!empty($headers[$header])) {
                return $headers[$header];
            }
        }

        return false;
    }
    /**
     * Is the request a Javascript XMLHttpRequest?
     *
     * Should work with Prototype/Script.aculo.us, possibly others.
     *
     * @return boolean
     */
    static function isXmlHttpRequest()
    {
        return (self::getHeader('X_REQUESTED_WITH') == 'XMLHttpRequest');
    }
}
