<?php
/**
 * @category View
 * @package Drupal MVC Example
 * @author Dmitry Sheiko <me@dsheiko.com>
 * @copyright 2010-2011 Dmitry Sheiko
 * @license GNU
 * @version $Revision$
 * @link http://dsheiko.com
 */

/**
 * Implements view
 */
class View_View {

    private $_vars;
    
    /**
     *
     * @param array $vars
     */
    public function  __construct($vars)
    {
        $this->_vars = $vars;
    }

    /**
     * Dispath and render requested application
     * 
     * @param string $request
     */
    public function dispatch($request)
    {
        $module = new Lib_Application();
        $widget = $module->run($this, $request);
        // Local namespace
        $view = $this;
        include TPL_PATH  . "/"  . $request . ".phtml";
    }

    /**
     * Accesses a helper object from within a script.
     *
     * @param string $name The helper name.
     * @param array $args The parameters for the helper.
     * @return string The result of the helper output.
     */
    public function __call($name, $args)
    {
        // Let's assume a view helper requested
        $suggestion  = 'View_Helper_' . ucfirst($name);
        if (class_exists($suggestion)) {
            return new $suggestion($this, $args);
        }
        return null;
    }

    /**
     * Get alias
     * @param string $name
     */
    public function  __get($name)
    {
        return $this->get($name);
    }

    /**
     * View getter
     *
     * @param string $name
     * @return mixed
     */
    public function  get($name)
    {
        if (isset($this->_vars[$name])) {
            return $this->_vars[$name];
        }
        return null;
    }
    /**
     * Set alias
     * @param string $name
     * @param mixed $value
     */
    public function  __set($name, $value)
    {
        return $this->set($name, $value);
    }

    /**
     * View setter
     *
     * @param string $name
     * @param mixed $value
     */
    public function  set($name, $value)
    {
        $this->_vars[$name] = $value;
    }
}